import AVFoundation
import Combine
import Observation

enum PlaybackState {
    case paused
    case playing
}

@Observable
final class Player {
    @ObservationIgnored private let player: AVPlayer

    @ObservationIgnored private var cancellables = Set<AnyCancellable>()
    @ObservationIgnored private let queue = DispatchQueue(label: "ch.defagos.observable-player")

    private(set) var playbackState: PlaybackState = .paused

    init(url: URL) {
        player = .init(url: url)
        configurePlaybackStatePublisher()
    }

    func configurePlaybackStatePublisher() {
        playbackStatePublisher()
            .weakAssign(to: \.playbackState, on: self)
            .store(in: &cancellables)
    }

    func playbackStatePublisher() -> AnyPublisher<PlaybackState, Never> {
        player.publisher(for: \.rate)
            .map { $0 == 0 ? .paused : .playing }
            .receive(on: queue)
            .eraseToAnyPublisher()
    }

    func togglePlayPause() {
        if player.rate != 0 {
            player.pause()
        }
        else {
            player.play()
        }
    }
}
