import Combine

extension Publisher where Failure == Never {
    /// Assigns each element from a publisher to a property on a weak object.
    ///
    /// - Parameters:
    ///   - keyPath: A key path that indicates the property to assign.
    ///   - object: The object that contains the property. The subscriber assigns the object’s property every time it
    ///     receives a new value.
    /// - Returns: An `AnyCancellable` instance which can be used to cancel the subscription.
    func weakAssign<T: AnyObject>(to keyPath: ReferenceWritableKeyPath<T, Output>, on object: T) -> AnyCancellable {
        sink { [weak object] value in
            object?[keyPath: keyPath] = value
        }
    }
}
